@tool
extends GridContainer

var current_position
var squares = {}

var black_turn: bool = true
var my_turn: bool = true
var game_over: bool = false

var targetable_positions: Array
var movable_to_positions: Array

var moves: Array

signal request_move(from: Vector2, to: Vector2)
signal move_completed
signal game_ended(winner: String)

# Called when the node enters the scene tree for the first time.
func _ready() -> void:
	columns = 5
	reset()


func reset() -> void:
	current_position = null
	for child in get_children():
		child.queue_free()
	add_child_squares()
	
	game_over = false
	
	targetable_positions.clear()
	movable_to_positions.clear()


func add_child_squares() -> void:
	var alternate = true
	for row in range(5):
		for col in range(5):
			var square = Square.new()
			square.custom_minimum_size = Vector2(64, 64)
			square.row = row
			square.col = col
			if alternate:
				square.color = Color.ALICE_BLUE
				square.normal_color = Color.ALICE_BLUE
			else:
				square.color = Color.WEB_GRAY
				square.normal_color = Color.WEB_GRAY
			if col == 2 and (row == 0 or row == 4):
				square.color = Color.DARK_GOLDENROD
				square.normal_color = Color.DARK_GOLDENROD
			if row == 0:
				if col == 2:
					var piece = Sprite2D.new()
					piece.texture = load("res://pieces/whiteKing.png")
					piece.flip_v = true
					piece.position.x += 32
					piece.position.y += 32
					square.add_child(piece)
					square.piece = Square.Piece_Type.WHITE_KING
				else:
					var piece = Sprite2D.new()
					piece.texture = load("res://pieces/whitePawn.png")
					piece.flip_v = true
					piece.position.x += 32
					piece.position.y += 32
					square.add_child(piece)
					square.piece = Square.Piece_Type.WHITE_PAWN
			elif row == 4:
				if col == 2:
					var piece = Sprite2D.new()
					piece.texture = load("res://pieces/blackKing.png")
					piece.position.x += 32
					piece.position.y += 32
					square.add_child(piece)
					square.piece = Square.Piece_Type.BLACK_KING
				else:
					var piece = Sprite2D.new()
					piece.texture = load("res://pieces/blackPawn.png")
					piece.position.x += 32
					piece.position.y += 32
					square.add_child(piece)
					square.piece = Square.Piece_Type.BLACK_PAWN
			else:
					square.piece = Square.Piece_Type.EMPTY
			alternate = not alternate
			square.connect("square_clicked", Callable(self, "select_square"))
			square.connect("piece_captured", Callable(self, "check_capture_victory"))
			add_child(square)
			squares[Vector2(col, row)] = square
			

func select_square(row, col, piece) -> void:
	if (not game_over and my_turn):
		if current_position == null:
			if match_turn(piece):
				current_position = Vector2(col, row)
				find_targetable_positions()
				color_targetable_positions()
				find_movable_positions()
				color_movable_positions()
				squares.get(current_position).color = Color.GREEN
		else:
			if match_turn(piece):
				squares.get(current_position).color = squares.get(current_position).normal_color
				current_position = Vector2(col, row)
				find_targetable_positions()
				color_targetable_positions()
				find_movable_positions()
				color_movable_positions()
				squares.get(current_position).color = Color.GREEN
			else:
				var target = Vector2(col, row)
				if validate_move(current_position, target):
					request_move.emit(current_position, target)


func move_piece(from: Vector2, to: Vector2) -> void:
	var from_square = squares.get(from)
	var to_square = squares.get(to)
	
	to_square.move_square_in(from_square)
	
	from_square.color = from_square.normal_color
	
	reset_targetable_color()
	targetable_positions.clear()
	
	reset_movable_color()
	movable_to_positions.clear()
	
	check_position_victory()
	
	moves = []
	
	current_position = null
	
	next_turn()


func match_turn(piece) -> bool:
	if black_turn:
		if multiplayer.get_unique_id() == 1:
			if (piece == Square.Piece_Type.BLACK_KING or piece == Square.Piece_Type.BLACK_PAWN):
				return true
			else:
				return false
		else:
			return false
	else:
		if multiplayer.get_unique_id() != 1:
			if (piece == Square.Piece_Type.WHITE_KING or piece == Square.Piece_Type.WHITE_PAWN):
				return true
			else:
				return false
		else:
			return false


func on_board(move: Vector2) -> bool:
	var pos_x = current_position.x + move.x
	var pos_y = current_position.y + move.y
	if (pos_x >= 0 and pos_x <= 4 and pos_y >= 0 and pos_y <= 4):
		return true
	else:
		return false
		

func find_targetable_positions() -> void:
	reset_targetable_color()
	
	targetable_positions.clear()
	
	for move in moves:
		if black_turn:
			move = -move
		if on_board(move):
			targetable_positions.append(current_position + move)

func color_targetable_positions() -> void:
	for pos in targetable_positions:
		squares.get(pos).color = Color.NAVY_BLUE


func reset_targetable_color() -> void:
	for pos in targetable_positions:
		squares.get(pos).color = squares.get(pos).normal_color
	

func find_movable_positions() -> void:
	reset_movable_color()
	
	movable_to_positions.clear()
	
	for pos in targetable_positions:
		var current_square = squares.get(current_position)
		var target_square = squares.get(pos)
		if (not current_square.color_match(target_square)):
			movable_to_positions.append(pos)
	

func color_movable_positions() -> void:
	for pos in movable_to_positions:
		squares.get(pos).color = Color.DODGER_BLUE


func reset_movable_color() -> void:
	for pos in movable_to_positions:
		squares.get(pos).color = squares.get(pos).normal_color
	

func validate_move(_from: Vector2, to: Vector2) -> bool:
	return movable_to_positions.has(to)


func check_capture_victory(piece: Square.Piece_Type) -> void:
	if (piece == Square.Piece_Type.BLACK_KING):
		game_over = true
		game_ended.emit("White wins, Black starts next")
	elif (piece == Square.Piece_Type.WHITE_KING):
		game_over = true	
		game_ended.emit("Black wins, White starts next")


func check_position_victory() -> void:
	if (squares.get(Vector2(2, 0)).piece == Square.Piece_Type.BLACK_KING):
		game_over = true
		game_ended.emit("Black wins, White starts next")
	elif (squares.get(Vector2(2, 4)).piece == Square.Piece_Type.WHITE_KING):
		game_over = true
		game_ended.emit("White wins, Black starts next")


func next_turn() -> void:
	black_turn = not black_turn
	move_completed.emit()


func set_moves(new_moves: Array) -> void:
	moves = new_moves
	print(current_position)
	if (current_position != null):
		find_targetable_positions()
		color_targetable_positions()
		find_movable_positions()
		color_movable_positions()

