extends Node


func _ready():
	MultiplayerManager.connect("player_connected", Callable(self, "manage_connects"))
	%Board.connect("request_move", Callable(self, "move_clients"))
	

func manage_connects(peer_id) -> void:
	print(peer_id)
	start()


func start():
	%StatusLabel.text = "Host and client joined! I'm peer %s" % multiplayer.get_unique_id()
	%JoinUI.hide()
	%Table.show()
	if multiplayer.get_unique_id() == 1:
		make_five_moves.rpc(Moves.get_five_moves())


func restart():
	remote_restart.rpc()


@rpc("any_peer", "call_local")
func remote_restart():
	%Table.reset()
	%Board.reset()
	start()
	%GameOverUI.hide()
	%Table.show()


func hide_ui() -> void:
	%JoinUI.hide()


func hide_buttons() -> void:
	%HostBox.hide()
	%JoinBox.hide()	


func hide_table() -> void:
	%Table.hide()


func show_game_over(winner: String) -> void:
	%GameOverUI.show()
	%Winner.text = winner


func _on_host_pressed() -> void:
	MultiplayerManager.SERVER_IP = %HostText.text
	var error = MultiplayerManager.host()
	if error:
		%StatusLabel.text = "Error hosting the game"
	else:
		#hide_buttons()
		%StatusLabel.text = "Hosting...\n Waiting for client to join..."


func _on_join_pressed() -> void:
	MultiplayerManager.SERVER_IP = %JoinText.text
	var error = MultiplayerManager.join()
	if error:
		%StatusLabel.text = "Error joining the game"
	else:
		#hide_buttons()
		%StatusLabel.text = "Joining...\n Waiting for host to start..."


func move_clients(from: Vector2, to: Vector2) -> void:
	remote_board_move.rpc(from, to)


@rpc("any_peer", "call_local")
func remote_board_move(from: Vector2, to: Vector2) -> void:
	%Board.move_piece(from, to)


func swap_client_tiles(player: int, index: int) -> void:
	#both boards will signal when the move is completed, but only one knows what
	#tile was actually selected (hiding "thinking" selections from the opponent)
	#so only rpc when the game knows which tile is which
	if index != -1:
		remote_tile_swap.rpc(player, index)


@rpc("any_peer", "call_local")
func remote_tile_swap(player: int, index: int) -> void:
	%Table.swap_tile(player, index)


@rpc("any_peer", "call_local")
func make_five_moves(moves: Array) -> void:
	var rotate: bool
	if multiplayer.get_unique_id() == 1:
		rotate = false
	else:
		rotate = true
	%Table.setup(moves, rotate)


func _on_board_game_ended(winner: String) -> void:
	hide_table()
	show_game_over(winner)
