extends VBoxContainer

var selected_tile: MoveTile
var move_tiles: Array

var P1Tiles: Array
var P2Tiles: Array
var UpcomingTile

#this'll need to hop out into the game manage/have some validation out there
var black_turn: bool = true

var rotated: bool = false

enum{PLAYER1, PLAYER2}
signal request_tile_swap(player: int, index: int)


func reset() -> void:
	selected_tile = null
	move_tiles.clear()
	P1Tiles.clear()
	P2Tiles.clear()
	UpcomingTile = null
	for child in %P2Moves.get_children():
		child.queue_free()
	for child in %P1Moves.get_children():
		child.queue_free()
	for child in %Upcoming.get_children():
		child.queue_free()


func setup(five_moves: Array, rotate: bool) -> void:
	var tile_scene = load("res://move_tile.tscn")
	var counter: int = 0
	for move_name in five_moves:
		var temp = tile_scene.instantiate()
		temp.move_name = move_name
		if counter < 2:
			temp.rotate_texture()
			P2Tiles.append(temp)
		elif counter == 2:
			UpcomingTile = temp
		else:
			P1Tiles.append(temp)
		temp.connect("move_tile_clicked", Callable(%Board, "set_moves"))
		temp.connect("move_tile_clicked", Callable(self, "set_tile").bind(temp))
		counter += 1
		parent_tiles()
		move_tiles.append(temp)
	set_selectable_tiles()
	if rotate and !rotated:
		rotate_me()
		rotated = true


func parent_tiles() -> void:
	for child in %P1Moves.get_children():
		%P1Moves.remove_child(child)
	for child in %P2Moves.get_children():
		%P2Moves.remove_child(child)
	for child in %Upcoming.get_children():
		%Upcoming.remove_child(child)
	
	for tile in P1Tiles:
		%P1Moves.add_child(tile)
	for tile in P2Tiles:
		%P2Moves.add_child(tile)
	if UpcomingTile:
		%Upcoming.add_child(UpcomingTile)
	
	
func set_tile(_moves: Array, tile: MoveTile) -> void:
	if (selected_tile):
		selected_tile.deselect()
	selected_tile = tile

func deselect_all_tiles() -> void:
	for tile in move_tiles:
		tile.deselect()


func set_selectable_tiles() -> void:
	if (black_turn):
		if (multiplayer.get_unique_id() == 1):
			for tile in P1Tiles:
				tile.selectable = true
			for tile in P2Tiles:
				tile.selectable = false
			UpcomingTile.selectable = false
		else:
			for tile in move_tiles:
				tile.selectable = false
	else:
		if (multiplayer.get_unique_id() != 1):
			for tile in P1Tiles:
				tile.selectable = false
			for tile in P2Tiles:
				tile.selectable = true
			UpcomingTile.selectable = false
		else:
			for tile in move_tiles:
				tile.selectable = false
				

func swap_tile(player: int, index: int) -> void:
	if player == PLAYER1:
		var temp = P1Tiles[index]
		P1Tiles[index] = UpcomingTile
		UpcomingTile = temp
		UpcomingTile.rotate_texture()
	else:
		var temp = P2Tiles[index]
		P2Tiles[index] = UpcomingTile
		UpcomingTile = temp
		UpcomingTile.rotate_texture()
	parent_tiles()
	selected_tile = null
	#turn transition
	black_turn = !black_turn
	set_selectable_tiles()


func _on_board_move_completed() -> void:
	print("Board move completed")
	
	deselect_all_tiles()
	
	if black_turn:
		request_tile_swap.emit(PLAYER1, P1Tiles.find(selected_tile))
	else:
		request_tile_swap.emit(PLAYER2, P2Tiles.find(selected_tile))


func rotate_me() -> void:
	pivot_offset = (size + Vector2(82, 164)) / 2 #oy vey
	rotation += PI 
